# TFG-Sonic

This repository contains the code to run Joint PPO and Rainbow with the availability to choose the optimizer. The available optimizers are: Adam, Adadelta, AdaGrad, NAG and Momentum.

## Prerequisites

* Python 3.7
* pip
* [virtualenv](https://gist.github.com/Geoyi/d9fab4f609e9f75941946be45000632b)

## Set up
0. Download the project
```bash
git clone git@github.com:xavi2411/Sonic-Baselines.git
cd Sonic-Baselines
```

1. Create a virtual environment
```bash
# Create the virtual environment
virtualenv <env-name> -p <path-to-python3.7>
# Activate the virtual environment
source <env-name>/bin/activate
# Deactivate the virtual environment
deactivate
```
Inside the environment python and pip can be used as `python`, `pip` without specifying the version, the virtual environment version will be used.

2. Install gym-retro
```bash
# Install gym-retro environment
pip install gym-retro
```
3. Get the Sonic games available on Steam:
* [Sonic The Hedgehog](https://store.steampowered.com/app/71113/Sonic_The_Hedgehog/)
* [Sonic The Hedgehog 2](https://store.steampowered.com/app/71163/Sonic_The_Hedgehog_2/)
* [Sonic 3 & Knuckles](https://store.steampowered.com/app/71162/Sonic_3__Knuckles/)

4. Download the ROMs of the games and import them
```bash
python -m retro.import.sega_classics
```
If the games are already installed, they can be imported directly.
```bash
python -m retro.import <path-to-steam-folder>
```
5. Install retro contest dependencies
```bash
git clone --recursive https://github.com/openai/retro-contest.git
pip install -e "retro-contest/support[docker,rest]"
```
6. Install tensorflow 1.14
```bash
pip install tensorflow-gpu==1.14 # If there's a GPU available
# or
pip install tensorflow==1.14
```
7. Install numpy and pandas 
```bash
pip install numpy
pip install pandas
```
8. Install matplotlib 
```bash
pip install matplotlib
```


## Run
The experiments consists on two parts: train and test. Both of them should be run in order to create the proper directory structure.

Then we have two python scripts to evaluate those experiments.
### Joint PPO
#### Training
We could simply run the `run.py` script:
```bash
python -W ignore run.py --alg ppo --optim adam --save_path RUNS
```
or we could run the `execute_ppo_trainings` script to execute the experiments with each optimizer:
```bash
./execute_ppo_trainings
```
#### Testing
We could simply run the `run.py` script:
```bash
python -W ignore run.py --alg ppo --load_path RUNS/ppo_training_1620984174 --test
```
or we could run the `execute_ppo_evals` script to execute the experiments with each optimizer but folders inside the script must be modified according to trainings:
```bash
./execute_ppo_evals
```
### Rainbow
#### Training
We could simply run the `run.py` script:
```bash
python -W ignore run.py --alg rainbow --optim adam --save_path RUNS
```
or we could run the `execute_rainbow_trainings` script to execute the experiments with each optimizer:
```bash
./execute_ppo_trainings
```
#### Testing
We could simply run the `run.py` script:
```bash
python -W ignore run.py --alg rainbow --load_path RUNS/rainbow_training_1620984174 --test
```
or we could run the `execute_rainbow_evals` script to execute the experiments with each optimizer but folders inside the script must be modified according to trainings:
```bash
./execute_rainbow_evals
```
### Evaluation Metrics
Run the `build_evaluation_metrics_ppo.py` or `build_evaluation_metrics_rainbow.py` according to the algorithm you want to get the metrics.

**Note:** A manually step is required, you should move each algorithm training folder to another directory and use that as argument.
```bash
python build_evaluation_metrics_ppo.py --dir ../ppo_results
```
```bash
python build_evaluation_metrics_rainbow.py --dir ../rainbow_results
```
### BK2 to MP4
At each Testing phase, a video is recorded in `bk2` format, in order to transform to mp4 run the following line (specifying the bk2 gile you want to use)
```bash
python -m retro.scripts.playback_movie SonicTheHedgehog-Genesis-GreenHillZone.Act2-000000.bk2
```