import numpy as np
import random

def explained_variance(ypred,y):
	"""
	Computes fraction of variance that ypred explains about y.
	Returns 1 - Var[y-ypred] / Var[y]

	interpretation:
		ev=0  =>  might as well have predicted zero
		ev=1  =>  perfect prediction
		ev<0  =>  worse than just predicting zero

	"""
	assert y.ndim == 1 and ypred.ndim == 1
	vary = np.var(y)
	return np.nan if vary==0 else 1 - np.var(y-ypred)/vary

def set_global_seeds(i):
	try:
		import MPI
		rank = MPI.COMM_WORLD.Get_rank()
	except ImportError:
		rank = 0

	myseed = i  + 1000 * rank if i is not None else None
	try:
		import tensorflow as tf
		tf.set_random_seed(myseed)
	except ImportError:
		pass
	np.random.seed(myseed)
	random.seed(myseed)